//$Id: CastFunction.java,v 1.4 2005/04/29 15:32:30 oneovthafew Exp $
package org.hibernate.dialect.function;

import java.util.List;

import org.hibernate.QueryException;
import org.hibernate.engine.Mapping;
import org.hibernate.engine.SessionFactoryImplementor;
import org.hibernate.mapping.Column;
import org.hibernate.type.Type;
import org.hibernate.type.TypeFactory;

/**
 * ANSI-SQL style <tt>cast(foo as type)</tt> where the type is
 * a Hibernate type
 * @author Gavin King
 */
public class CastFunction implements SQLFunction {

	public Type getReturnType(Type columnType, Mapping mapping) throws QueryException {
		//TODO: pass in the arguments here...
		return null; //it needs to be the argument type
	}

	public boolean hasArguments() {
		return true;
	}

	public boolean hasParenthesesIfNoArguments() {
		return true;
	}

	public String render(List args, SessionFactoryImplementor factory) throws QueryException {
		if ( args.size()!=2 ) {
			throw new QueryException("cast() requires two arguments");
		}
		String type = (String) args.get(1);
		int[] sqlTypeCodes = TypeFactory.heuristicType(type).sqlTypes(factory);
		if ( sqlTypeCodes.length!=1 ) {
			throw new QueryException("invalid Hibernate type for cast()");
		}
		String sqlType = factory.getDialect().getTypeName( 
				sqlTypeCodes[0], 
				Column.DEFAULT_LENGTH, 
				Column.DEFAULT_PRECISION, 
				Column.DEFAULT_SCALE 
			);
		if (sqlType==null) {
			//TODO: never reached, since getTypeName() actually throws an exception!
			sqlType = type;
		}
		/*else {
			//trim off the length/precision/scale
			int loc = sqlType.indexOf('(');
			if (loc>-1) {
				sqlType = sqlType.substring(0, loc);
			}
		}*/
		return "cast(" + args.get(0) + " as " + sqlType + ')';
	}

}
