// $Id: StandardProperty.java,v 1.4 2005/04/03 15:55:03 oneovthafew Exp $
package org.hibernate.tuple;

import org.hibernate.engine.Cascades;
import org.hibernate.type.Type;

/**
 * Represents a basic property within the Hibernate runtime-metamodel.
 *
 * @author Steve Ebersole
 */
public class StandardProperty extends Property {

	private boolean lazy;
	private boolean insertable;
	private boolean updateable;
	private boolean nullable;
	private boolean dirtyCheckable;
	private boolean versionable;
	private Cascades.CascadeStyle cascadeStyle;

	/**
	 * Constructs StandardProperty instances.
	 *
	 * @param name The name by which the property can be referenced within
	 * its owner.
	 * @param node The node name to use for XML-based representation of this
	 * property.
	 * @param type The Hibernate Type of this property.
	 * @param lazy Should this property be handled lazily?
	 * @param insertable Is this property an insertable value?
	 * @param updateable Is this property an updateable value?
	 * @param nullable Is this property a nullable value?
	 * @param checkable Is this property a checkable value?
	 * @param versionable Is this property a versionable value?
	 * @param cascadeStyle The cascade style for this property's value.
	 */
	public StandardProperty(
	        String name,
	        String node,
	        Type type,
	        boolean lazy,
	        boolean insertable,
	        boolean updateable,
	        boolean nullable,
	        boolean checkable,
	        boolean versionable,
	        Cascades.CascadeStyle cascadeStyle) {
		super(name, node, type);
		this.lazy = lazy;
		this.insertable = insertable;
		this.updateable = updateable;
		this.nullable = nullable;
		this.dirtyCheckable = checkable;
		this.versionable = versionable;
		this.cascadeStyle = cascadeStyle;
	}

	public boolean isLazy() {
		return lazy;
	}

	public void setLazy(boolean lazy) {
		this.lazy = lazy;
	}

	public boolean isInsertable() {
		return insertable;
	}

	public void setInsertable(boolean insertable) {
		this.insertable = insertable;
	}

	public boolean isUpdateable() {
		return updateable;
	}

	public void setUpdateable(boolean updateable) {
		this.updateable = updateable;
	}

	public boolean isNullable() {
		return nullable;
	}

	public void setNullable(boolean nullable) {
		this.nullable = nullable;
	}

	public boolean isDirtyCheckable(boolean hasUninitializedProperties) {
		return isDirtyCheckable() && ( !hasUninitializedProperties || !isLazy() );
	}

	public boolean isDirtyCheckable() {
		return dirtyCheckable;
	}

	public void setDirtyCheckable(boolean checkable) {
		this.dirtyCheckable = checkable;
	}

	public boolean isVersionable() {
		return versionable;
	}

	public void setVersionable(boolean versionable) {
		this.versionable = versionable;
	}

	public Cascades.CascadeStyle getCascadeStyle() {
		return cascadeStyle;
	}

	public void setCascadeStyle(Cascades.CascadeStyle cascadeStyle) {
		this.cascadeStyle = cascadeStyle;
	}
}
